# This file was auto-generated by Fern from our API Definition.

import contextlib
import json
import typing
from json.decoder import JSONDecodeError
from logging import warning

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.http_sse._api import EventSource
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.gone_error import GoneError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.event_stream_cloud_event_connection_created import EventStreamCloudEventConnectionCreated
from ..types.event_stream_cloud_event_connection_deleted import EventStreamCloudEventConnectionDeleted
from ..types.event_stream_cloud_event_connection_updated import EventStreamCloudEventConnectionUpdated
from ..types.event_stream_cloud_event_error_message import EventStreamCloudEventErrorMessage
from ..types.event_stream_cloud_event_group_created import EventStreamCloudEventGroupCreated
from ..types.event_stream_cloud_event_group_deleted import EventStreamCloudEventGroupDeleted
from ..types.event_stream_cloud_event_group_member_added import EventStreamCloudEventGroupMemberAdded
from ..types.event_stream_cloud_event_group_member_deleted import EventStreamCloudEventGroupMemberDeleted
from ..types.event_stream_cloud_event_group_role_assigned import EventStreamCloudEventGroupRoleAssigned
from ..types.event_stream_cloud_event_group_role_deleted import EventStreamCloudEventGroupRoleDeleted
from ..types.event_stream_cloud_event_group_updated import EventStreamCloudEventGroupUpdated
from ..types.event_stream_cloud_event_offset_only_message import EventStreamCloudEventOffsetOnlyMessage
from ..types.event_stream_cloud_event_org_connection_added import EventStreamCloudEventOrgConnectionAdded
from ..types.event_stream_cloud_event_org_connection_removed import EventStreamCloudEventOrgConnectionRemoved
from ..types.event_stream_cloud_event_org_connection_updated import EventStreamCloudEventOrgConnectionUpdated
from ..types.event_stream_cloud_event_org_created import EventStreamCloudEventOrgCreated
from ..types.event_stream_cloud_event_org_deleted import EventStreamCloudEventOrgDeleted
from ..types.event_stream_cloud_event_org_group_role_assigned import EventStreamCloudEventOrgGroupRoleAssigned
from ..types.event_stream_cloud_event_org_group_role_deleted import EventStreamCloudEventOrgGroupRoleDeleted
from ..types.event_stream_cloud_event_org_member_added import EventStreamCloudEventOrgMemberAdded
from ..types.event_stream_cloud_event_org_member_deleted import EventStreamCloudEventOrgMemberDeleted
from ..types.event_stream_cloud_event_org_member_role_assigned import EventStreamCloudEventOrgMemberRoleAssigned
from ..types.event_stream_cloud_event_org_member_role_deleted import EventStreamCloudEventOrgMemberRoleDeleted
from ..types.event_stream_cloud_event_org_updated import EventStreamCloudEventOrgUpdated
from ..types.event_stream_cloud_event_user_created import EventStreamCloudEventUserCreated
from ..types.event_stream_cloud_event_user_deleted import EventStreamCloudEventUserDeleted
from ..types.event_stream_cloud_event_user_updated import EventStreamCloudEventUserUpdated
from ..types.event_stream_subscribe_events_event_type_enum import EventStreamSubscribeEventsEventTypeEnum
from ..types.event_stream_subscribe_events_response_content import EventStreamSubscribeEventsResponseContent
from pydantic import ValidationError


class RawEventsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    @contextlib.contextmanager
    def subscribe(
        self,
        *,
        from_: typing.Optional[str] = None,
        from_timestamp: typing.Optional[str] = None,
        event_type: typing.Optional[
            typing.Union[
                EventStreamSubscribeEventsEventTypeEnum, typing.Sequence[EventStreamSubscribeEventsEventTypeEnum]
            ]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[HttpResponse[typing.Iterator[EventStreamSubscribeEventsResponseContent]]]:
        """
        Subscribe to events via Server-Sent Events (SSE)

        Parameters
        ----------
        from_ : typing.Optional[str]
            Opaque token representing position in the stream. If not provided, stream will start from the latest events.

        from_timestamp : typing.Optional[str]
            RFC-3339 timestamp indicating where to start streaming events from. This should only be used on the initial query when a cursor may not be available. Subsequent requests should use the cursor (from) as it will be more accurate.

        event_type : typing.Optional[typing.Union[EventStreamSubscribeEventsEventTypeEnum, typing.Sequence[EventStreamSubscribeEventsEventTypeEnum]]]
            Event type(s) to listen for. Specify multiple times for multiple types (e.g., ?event_type=user.created&event_type=user.updated). If not provided, all event types will be streamed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.Iterator[HttpResponse[typing.Iterator[EventStreamSubscribeEventsResponseContent]]]
            Event stream successfully established.
        """
        with self._client_wrapper.httpx_client.stream(
            "events",
            method="GET",
            params={
                "from": from_,
                "from_timestamp": from_timestamp,
                "event_type": event_type,
            },
            request_options=request_options,
        ) as _response:

            def _stream() -> HttpResponse[typing.Iterator[EventStreamSubscribeEventsResponseContent]]:
                try:
                    if 200 <= _response.status_code < 300:

                        def _iter():
                            _event_source = EventSource(_response)
                            for _sse in _event_source.iter_sse():
                                if _sse.data == None:
                                    return
                                if _sse.event == "connection.created":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventConnectionCreated,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventConnectionCreated,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'connection.created': {e}, sse: {_sse!r}")
                                elif _sse.event == "connection.deleted":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventConnectionDeleted,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventConnectionDeleted,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'connection.deleted': {e}, sse: {_sse!r}")
                                elif _sse.event == "connection.updated":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventConnectionUpdated,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventConnectionUpdated,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'connection.updated': {e}, sse: {_sse!r}")
                                elif _sse.event == "group.created":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventGroupCreated,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventGroupCreated,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'group.created': {e}, sse: {_sse!r}")
                                elif _sse.event == "group.deleted":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventGroupDeleted,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventGroupDeleted,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'group.deleted': {e}, sse: {_sse!r}")
                                elif _sse.event == "group.member.added":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventGroupMemberAdded,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventGroupMemberAdded,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'group.member.added': {e}, sse: {_sse!r}")
                                elif _sse.event == "group.member.deleted":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventGroupMemberDeleted,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventGroupMemberDeleted,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'group.member.deleted': {e}, sse: {_sse!r}")
                                elif _sse.event == "group.role.assigned":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventGroupRoleAssigned,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventGroupRoleAssigned,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'group.role.assigned': {e}, sse: {_sse!r}")
                                elif _sse.event == "group.role.deleted":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventGroupRoleDeleted,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventGroupRoleDeleted,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'group.role.deleted': {e}, sse: {_sse!r}")
                                elif _sse.event == "group.updated":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventGroupUpdated,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventGroupUpdated,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'group.updated': {e}, sse: {_sse!r}")
                                elif _sse.event == "organization.connection.added":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgConnectionAdded,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgConnectionAdded,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(
                                            f"Failed to parse SSE event 'organization.connection.added': {e}, sse: {_sse!r}"
                                        )
                                elif _sse.event == "organization.connection.removed":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgConnectionRemoved,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgConnectionRemoved,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(
                                            f"Failed to parse SSE event 'organization.connection.removed': {e}, sse: {_sse!r}"
                                        )
                                elif _sse.event == "organization.connection.updated":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgConnectionUpdated,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgConnectionUpdated,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(
                                            f"Failed to parse SSE event 'organization.connection.updated': {e}, sse: {_sse!r}"
                                        )
                                elif _sse.event == "organization.created":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgCreated,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgCreated,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'organization.created': {e}, sse: {_sse!r}")
                                elif _sse.event == "organization.deleted":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgDeleted,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgDeleted,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'organization.deleted': {e}, sse: {_sse!r}")
                                elif _sse.event == "organization.group.role.assigned":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgGroupRoleAssigned,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgGroupRoleAssigned,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(
                                            f"Failed to parse SSE event 'organization.group.role.assigned': {e}, sse: {_sse!r}"
                                        )
                                elif _sse.event == "organization.group.role.deleted":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgGroupRoleDeleted,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgGroupRoleDeleted,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(
                                            f"Failed to parse SSE event 'organization.group.role.deleted': {e}, sse: {_sse!r}"
                                        )
                                elif _sse.event == "organization.member.added":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgMemberAdded,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgMemberAdded,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(
                                            f"Failed to parse SSE event 'organization.member.added': {e}, sse: {_sse!r}"
                                        )
                                elif _sse.event == "organization.member.deleted":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgMemberDeleted,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgMemberDeleted,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(
                                            f"Failed to parse SSE event 'organization.member.deleted': {e}, sse: {_sse!r}"
                                        )
                                elif _sse.event == "organization.member.role.assigned":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgMemberRoleAssigned,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgMemberRoleAssigned,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(
                                            f"Failed to parse SSE event 'organization.member.role.assigned': {e}, sse: {_sse!r}"
                                        )
                                elif _sse.event == "organization.member.role.deleted":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgMemberRoleDeleted,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgMemberRoleDeleted,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(
                                            f"Failed to parse SSE event 'organization.member.role.deleted': {e}, sse: {_sse!r}"
                                        )
                                elif _sse.event == "organization.updated":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgUpdated,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgUpdated,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'organization.updated': {e}, sse: {_sse!r}")
                                elif _sse.event == "user.created":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventUserCreated,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventUserCreated,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'user.created': {e}, sse: {_sse!r}")
                                elif _sse.event == "user.deleted":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventUserDeleted,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventUserDeleted,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'user.deleted': {e}, sse: {_sse!r}")
                                elif _sse.event == "user.updated":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventUserUpdated,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventUserUpdated,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'user.updated': {e}, sse: {_sse!r}")
                                elif _sse.event == "error":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventErrorMessage,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventErrorMessage,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'error': {e}, sse: {_sse!r}")
                                elif _sse.event == "offset-only":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOffsetOnlyMessage,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOffsetOnlyMessage,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'offset-only': {e}, sse: {_sse!r}")
                            return

                        return HttpResponse(response=_response, data=_iter())
                    _response.read()
                    if _response.status_code == 400:
                        raise BadRequestError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 401:
                        raise UnauthorizedError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 403:
                        raise ForbiddenError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 404:
                        raise NotFoundError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 410:
                        raise GoneError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 429:
                        raise TooManyRequestsError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                except ValidationError as e:
                    raise ParsingError(
                        status_code=_response.status_code,
                        headers=dict(_response.headers),
                        body=_response.json(),
                        cause=e,
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield _stream()


class AsyncRawEventsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    @contextlib.asynccontextmanager
    async def subscribe(
        self,
        *,
        from_: typing.Optional[str] = None,
        from_timestamp: typing.Optional[str] = None,
        event_type: typing.Optional[
            typing.Union[
                EventStreamSubscribeEventsEventTypeEnum, typing.Sequence[EventStreamSubscribeEventsEventTypeEnum]
            ]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[EventStreamSubscribeEventsResponseContent]]]:
        """
        Subscribe to events via Server-Sent Events (SSE)

        Parameters
        ----------
        from_ : typing.Optional[str]
            Opaque token representing position in the stream. If not provided, stream will start from the latest events.

        from_timestamp : typing.Optional[str]
            RFC-3339 timestamp indicating where to start streaming events from. This should only be used on the initial query when a cursor may not be available. Subsequent requests should use the cursor (from) as it will be more accurate.

        event_type : typing.Optional[typing.Union[EventStreamSubscribeEventsEventTypeEnum, typing.Sequence[EventStreamSubscribeEventsEventTypeEnum]]]
            Event type(s) to listen for. Specify multiple times for multiple types (e.g., ?event_type=user.created&event_type=user.updated). If not provided, all event types will be streamed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.AsyncIterator[AsyncHttpResponse[typing.AsyncIterator[EventStreamSubscribeEventsResponseContent]]]
            Event stream successfully established.
        """
        async with self._client_wrapper.httpx_client.stream(
            "events",
            method="GET",
            params={
                "from": from_,
                "from_timestamp": from_timestamp,
                "event_type": event_type,
            },
            request_options=request_options,
        ) as _response:

            async def _stream() -> AsyncHttpResponse[typing.AsyncIterator[EventStreamSubscribeEventsResponseContent]]:
                try:
                    if 200 <= _response.status_code < 300:

                        async def _iter():
                            _event_source = EventSource(_response)
                            async for _sse in _event_source.aiter_sse():
                                if _sse.data == None:
                                    return
                                if _sse.event == "connection.created":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventConnectionCreated,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventConnectionCreated,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'connection.created': {e}, sse: {_sse!r}")
                                elif _sse.event == "connection.deleted":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventConnectionDeleted,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventConnectionDeleted,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'connection.deleted': {e}, sse: {_sse!r}")
                                elif _sse.event == "connection.updated":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventConnectionUpdated,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventConnectionUpdated,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'connection.updated': {e}, sse: {_sse!r}")
                                elif _sse.event == "group.created":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventGroupCreated,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventGroupCreated,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'group.created': {e}, sse: {_sse!r}")
                                elif _sse.event == "group.deleted":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventGroupDeleted,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventGroupDeleted,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'group.deleted': {e}, sse: {_sse!r}")
                                elif _sse.event == "group.member.added":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventGroupMemberAdded,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventGroupMemberAdded,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'group.member.added': {e}, sse: {_sse!r}")
                                elif _sse.event == "group.member.deleted":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventGroupMemberDeleted,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventGroupMemberDeleted,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'group.member.deleted': {e}, sse: {_sse!r}")
                                elif _sse.event == "group.role.assigned":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventGroupRoleAssigned,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventGroupRoleAssigned,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'group.role.assigned': {e}, sse: {_sse!r}")
                                elif _sse.event == "group.role.deleted":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventGroupRoleDeleted,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventGroupRoleDeleted,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'group.role.deleted': {e}, sse: {_sse!r}")
                                elif _sse.event == "group.updated":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventGroupUpdated,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventGroupUpdated,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'group.updated': {e}, sse: {_sse!r}")
                                elif _sse.event == "organization.connection.added":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgConnectionAdded,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgConnectionAdded,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(
                                            f"Failed to parse SSE event 'organization.connection.added': {e}, sse: {_sse!r}"
                                        )
                                elif _sse.event == "organization.connection.removed":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgConnectionRemoved,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgConnectionRemoved,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(
                                            f"Failed to parse SSE event 'organization.connection.removed': {e}, sse: {_sse!r}"
                                        )
                                elif _sse.event == "organization.connection.updated":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgConnectionUpdated,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgConnectionUpdated,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(
                                            f"Failed to parse SSE event 'organization.connection.updated': {e}, sse: {_sse!r}"
                                        )
                                elif _sse.event == "organization.created":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgCreated,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgCreated,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'organization.created': {e}, sse: {_sse!r}")
                                elif _sse.event == "organization.deleted":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgDeleted,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgDeleted,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'organization.deleted': {e}, sse: {_sse!r}")
                                elif _sse.event == "organization.group.role.assigned":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgGroupRoleAssigned,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgGroupRoleAssigned,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(
                                            f"Failed to parse SSE event 'organization.group.role.assigned': {e}, sse: {_sse!r}"
                                        )
                                elif _sse.event == "organization.group.role.deleted":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgGroupRoleDeleted,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgGroupRoleDeleted,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(
                                            f"Failed to parse SSE event 'organization.group.role.deleted': {e}, sse: {_sse!r}"
                                        )
                                elif _sse.event == "organization.member.added":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgMemberAdded,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgMemberAdded,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(
                                            f"Failed to parse SSE event 'organization.member.added': {e}, sse: {_sse!r}"
                                        )
                                elif _sse.event == "organization.member.deleted":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgMemberDeleted,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgMemberDeleted,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(
                                            f"Failed to parse SSE event 'organization.member.deleted': {e}, sse: {_sse!r}"
                                        )
                                elif _sse.event == "organization.member.role.assigned":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgMemberRoleAssigned,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgMemberRoleAssigned,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(
                                            f"Failed to parse SSE event 'organization.member.role.assigned': {e}, sse: {_sse!r}"
                                        )
                                elif _sse.event == "organization.member.role.deleted":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgMemberRoleDeleted,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgMemberRoleDeleted,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(
                                            f"Failed to parse SSE event 'organization.member.role.deleted': {e}, sse: {_sse!r}"
                                        )
                                elif _sse.event == "organization.updated":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOrgUpdated,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOrgUpdated,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'organization.updated': {e}, sse: {_sse!r}")
                                elif _sse.event == "user.created":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventUserCreated,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventUserCreated,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'user.created': {e}, sse: {_sse!r}")
                                elif _sse.event == "user.deleted":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventUserDeleted,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventUserDeleted,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'user.deleted': {e}, sse: {_sse!r}")
                                elif _sse.event == "user.updated":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventUserUpdated,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventUserUpdated,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'user.updated': {e}, sse: {_sse!r}")
                                elif _sse.event == "error":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventErrorMessage,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventErrorMessage,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'error': {e}, sse: {_sse!r}")
                                elif _sse.event == "offset-only":
                                    try:
                                        yield typing.cast(
                                            EventStreamCloudEventOffsetOnlyMessage,
                                            parse_obj_as(
                                                type_=EventStreamCloudEventOffsetOnlyMessage,  # type: ignore
                                                object_=json.loads(_sse.data),
                                            ),
                                        )
                                    except Exception as e:
                                        warning(f"Failed to parse SSE event 'offset-only': {e}, sse: {_sse!r}")
                            return

                        return AsyncHttpResponse(response=_response, data=_iter())
                    await _response.aread()
                    if _response.status_code == 400:
                        raise BadRequestError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 401:
                        raise UnauthorizedError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 403:
                        raise ForbiddenError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 404:
                        raise NotFoundError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 410:
                        raise GoneError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    if _response.status_code == 429:
                        raise TooManyRequestsError(
                            headers=dict(_response.headers),
                            body=typing.cast(
                                typing.Any,
                                parse_obj_as(
                                    type_=typing.Any,  # type: ignore
                                    object_=_response.json(),
                                ),
                            ),
                        )
                    _response_json = _response.json()
                except JSONDecodeError:
                    raise ApiError(
                        status_code=_response.status_code, headers=dict(_response.headers), body=_response.text
                    )
                except ValidationError as e:
                    raise ParsingError(
                        status_code=_response.status_code,
                        headers=dict(_response.headers),
                        body=_response.json(),
                        cause=e,
                    )
                raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

            yield await _stream()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .connection_password_options_complexity import ConnectionPasswordOptionsComplexity
from .connection_password_options_dictionary import ConnectionPasswordOptionsDictionary
from .connection_password_options_history import ConnectionPasswordOptionsHistory
from .connection_password_options_profile_data import ConnectionPasswordOptionsProfileData


class ConnectionPasswordOptions(UniversalBaseModel):
    """
    Password policy options for flexible password policy configuration
    """

    complexity: typing.Optional[ConnectionPasswordOptionsComplexity] = None
    dictionary: typing.Optional[ConnectionPasswordOptionsDictionary] = None
    history: typing.Optional[ConnectionPasswordOptionsHistory] = None
    profile_data: typing.Optional[ConnectionPasswordOptionsProfileData] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .connection_https_url_with_http_fallback_255 import ConnectionHttpsUrlWithHttpFallback255

ConnectionRecipientUrlSaml = ConnectionHttpsUrlWithHttpFallback255
"""
The URL where Auth0 will send SAML authentication requests (the Identity Provider's SSO URL). Must be a valid HTTPS URL.
"""



# This file was auto-generated by Fern from our API Definition.

from .connection_metadata_xml import ConnectionMetadataXml

ConnectionMetadataXmlAdfs = ConnectionMetadataXml
"""
Inline XML alternative to 'adfs_server'. Cannot be set together with 'adfs_server'.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

ResourceServerTokenDialectResponseEnum = typing.Union[
    typing.Literal["access_token", "access_token_authz", "rfc9068_profile", "rfc9068_profile_authz"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .flow_action_auth_0_create_user import FlowActionAuth0CreateUser
from .flow_action_auth_0_get_user import FlowActionAuth0GetUser
from .flow_action_auth_0_make_call import FlowActionAuth0MakeCall
from .flow_action_auth_0_send_email import FlowActionAuth0SendEmail
from .flow_action_auth_0_send_request import FlowActionAuth0SendRequest
from .flow_action_auth_0_send_sms import FlowActionAuth0SendSms
from .flow_action_auth_0_update_user import FlowActionAuth0UpdateUser

FlowActionAuth0 = typing.Union[
    FlowActionAuth0CreateUser,
    FlowActionAuth0GetUser,
    FlowActionAuth0UpdateUser,
    FlowActionAuth0SendRequest,
    FlowActionAuth0SendEmail,
    FlowActionAuth0SendSms,
    FlowActionAuth0MakeCall,
]



# This file was auto-generated by Fern from our API Definition.

import typing

ClientOrganizationRequireBehaviorEnum = typing.Union[
    typing.Literal["no_prompt", "pre_login_prompt", "post_login_prompt"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

ClientTokenExchangeTypeEnum = typing.Union[
    typing.Literal["custom_authentication", "on_behalf_of_token_exchange"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .flow_action_pipedrive_add_deal import FlowActionPipedriveAddDeal
from .flow_action_pipedrive_add_organization import FlowActionPipedriveAddOrganization
from .flow_action_pipedrive_add_person import FlowActionPipedriveAddPerson

FlowActionPipedrive = typing.Union[
    FlowActionPipedriveAddDeal, FlowActionPipedriveAddOrganization, FlowActionPipedriveAddPerson
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .connected_account_access_type_enum import ConnectedAccountAccessTypeEnum


class ConnectedAccount(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    The unique identifier for the connected account.
    """

    connection: str = pydantic.Field()
    """
    The name of the connection associated with the account.
    """

    connection_id: str = pydantic.Field()
    """
    The unique identifier of the connection associated with the account.
    """

    strategy: str = pydantic.Field()
    """
    The authentication strategy used by the connection.
    """

    access_type: ConnectedAccountAccessTypeEnum
    scopes: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The scopes granted for this connected account.
    """

    created_at: dt.datetime = pydantic.Field()
    """
    ISO 8601 timestamp when the connected account was created.
    """

    expires_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    ISO 8601 timestamp when the connected account expires.
    """

    organization_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The identifier of the organization associated with the connected account.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionOptions = typing.Dict[str, typing.Any]
"""
In order to return options in the response, the `read:connections_options` scope must be present
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class MdlPresentationProperties(UniversalBaseModel):
    family_name: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Family Name
    """

    given_name: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Given Name
    """

    birth_date: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Birth Date
    """

    issue_date: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Issue Date
    """

    expiry_date: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Expiry Date
    """

    issuing_country: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Issuing Country
    """

    issuing_authority: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Issuing Authority
    """

    portrait: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Portrait
    """

    driving_privileges: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Driving Privileges
    """

    resident_address: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Resident Address
    """

    portrait_capture_date: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Portrait Capture Date
    """

    age_in_years: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Age in Years
    """

    age_birth_year: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Age Birth Year
    """

    issuing_jurisdiction: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Issuing Jurisdiction
    """

    nationality: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Nationality
    """

    resident_city: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Resident City
    """

    resident_state: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Resident State
    """

    resident_postal_code: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Resident Postal Code
    """

    resident_country: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Resident Country
    """

    family_name_national_character: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Family Name National Character
    """

    given_name_national_character: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Given Name National Character
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .connection_disable_signup import ConnectionDisableSignup

ConnectionDisableSignupSms = ConnectionDisableSignup
"""
Controls whether new user signups are allowed via SMS authentication
"""



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...errors.bad_request_error import BadRequestError
from ...errors.forbidden_error import ForbiddenError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.brute_force_protection_mode_enum import BruteForceProtectionModeEnum
from ...types.brute_force_protection_shields_enum import BruteForceProtectionShieldsEnum
from ...types.get_brute_force_settings_response_content import GetBruteForceSettingsResponseContent
from ...types.update_brute_force_settings_response_content import UpdateBruteForceSettingsResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawBruteForceProtectionClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetBruteForceSettingsResponseContent]:
        """
        Retrieve details of the Brute-force Protection configuration of your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetBruteForceSettingsResponseContent]
            Brute force configuration successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "attack-protection/brute-force-protection",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBruteForceSettingsResponseContent,
                    parse_obj_as(
                        type_=GetBruteForceSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        *,
        enabled: typing.Optional[bool] = OMIT,
        shields: typing.Optional[typing.Sequence[BruteForceProtectionShieldsEnum]] = OMIT,
        allowlist: typing.Optional[typing.Sequence[str]] = OMIT,
        mode: typing.Optional[BruteForceProtectionModeEnum] = OMIT,
        max_attempts: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateBruteForceSettingsResponseContent]:
        """
        Update the Brute-force Protection configuration of your tenant.

        Parameters
        ----------
        enabled : typing.Optional[bool]
            Whether or not brute force attack protections are active.

        shields : typing.Optional[typing.Sequence[BruteForceProtectionShieldsEnum]]
            Action to take when a brute force protection threshold is violated.
                    Possible values: <code>block</code>, <code>user_notification</code>.

        allowlist : typing.Optional[typing.Sequence[str]]
            List of trusted IP addresses that will not have attack protection enforced against them.

        mode : typing.Optional[BruteForceProtectionModeEnum]

        max_attempts : typing.Optional[int]
            Maximum number of unsuccessful attempts.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateBruteForceSettingsResponseContent]
            Brute force configuration successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            "attack-protection/brute-force-protection",
            method="PATCH",
            json={
                "enabled": enabled,
                "shields": shields,
                "allowlist": allowlist,
                "mode": mode,
                "max_attempts": max_attempts,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateBruteForceSettingsResponseContent,
                    parse_obj_as(
                        type_=UpdateBruteForceSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawBruteForceProtectionClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetBruteForceSettingsResponseContent]:
        """
        Retrieve details of the Brute-force Protection configuration of your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetBruteForceSettingsResponseContent]
            Brute force configuration successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "attack-protection/brute-force-protection",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBruteForceSettingsResponseContent,
                    parse_obj_as(
                        type_=GetBruteForceSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        *,
        enabled: typing.Optional[bool] = OMIT,
        shields: typing.Optional[typing.Sequence[BruteForceProtectionShieldsEnum]] = OMIT,
        allowlist: typing.Optional[typing.Sequence[str]] = OMIT,
        mode: typing.Optional[BruteForceProtectionModeEnum] = OMIT,
        max_attempts: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateBruteForceSettingsResponseContent]:
        """
        Update the Brute-force Protection configuration of your tenant.

        Parameters
        ----------
        enabled : typing.Optional[bool]
            Whether or not brute force attack protections are active.

        shields : typing.Optional[typing.Sequence[BruteForceProtectionShieldsEnum]]
            Action to take when a brute force protection threshold is violated.
                    Possible values: <code>block</code>, <code>user_notification</code>.

        allowlist : typing.Optional[typing.Sequence[str]]
            List of trusted IP addresses that will not have attack protection enforced against them.

        mode : typing.Optional[BruteForceProtectionModeEnum]

        max_attempts : typing.Optional[int]
            Maximum number of unsuccessful attempts.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateBruteForceSettingsResponseContent]
            Brute force configuration successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "attack-protection/brute-force-protection",
            method="PATCH",
            json={
                "enabled": enabled,
                "shields": shields,
                "allowlist": allowlist,
                "mode": mode,
                "max_attempts": max_attempts,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateBruteForceSettingsResponseContent,
                    parse_obj_as(
                        type_=UpdateBruteForceSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.create_directory_provisioning_request_content import CreateDirectoryProvisioningRequestContent
from ...types.create_directory_provisioning_response_content import CreateDirectoryProvisioningResponseContent
from ...types.directory_provisioning import DirectoryProvisioning
from ...types.get_directory_provisioning_default_mapping_response_content import (
    GetDirectoryProvisioningDefaultMappingResponseContent,
)
from ...types.get_directory_provisioning_response_content import GetDirectoryProvisioningResponseContent
from ...types.list_directory_provisionings_response_content import ListDirectoryProvisioningsResponseContent
from ...types.list_synchronized_groups_response_content import ListSynchronizedGroupsResponseContent
from ...types.synchronized_group_payload import SynchronizedGroupPayload
from ...types.update_directory_provisioning_request_content import UpdateDirectoryProvisioningRequestContent
from ...types.update_directory_provisioning_response_content import UpdateDirectoryProvisioningResponseContent
from .raw_client import AsyncRawDirectoryProvisioningClient, RawDirectoryProvisioningClient

if typing.TYPE_CHECKING:
    from .synchronizations.client import AsyncSynchronizationsClient, SynchronizationsClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DirectoryProvisioningClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDirectoryProvisioningClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._synchronizations: typing.Optional[SynchronizationsClient] = None

    @property
    def with_raw_response(self) -> RawDirectoryProvisioningClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDirectoryProvisioningClient
        """
        return self._raw_client

    def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[DirectoryProvisioning, ListDirectoryProvisioningsResponseContent]:
        """
        Retrieve a list of directory provisioning configurations of a tenant.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[DirectoryProvisioning, ListDirectoryProvisioningsResponseContent]
            The tenant's directory provisioning configuration. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.connections.directory_provisioning.list(
            from_="from",
            take=1,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(from_=from_, take=take, request_options=request_options)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetDirectoryProvisioningResponseContent:
        """
        Retrieve the directory provisioning configuration of a connection.

        Parameters
        ----------
        id : str
            The id of the connection to retrieve its directory provisioning configuration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDirectoryProvisioningResponseContent
            The connection's directory provisioning configuration. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.directory_provisioning.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def create(
        self,
        id: str,
        *,
        request: typing.Optional[CreateDirectoryProvisioningRequestContent] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateDirectoryProvisioningResponseContent:
        """
        Create a directory provisioning configuration for a connection.

        Parameters
        ----------
        id : str
            The id of the connection to create its directory provisioning configuration

        request : typing.Optional[CreateDirectoryProvisioningRequestContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateDirectoryProvisioningResponseContent
            The connection's directory provisioning configuration was created. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        from auth0 import Auth0, CreateDirectoryProvisioningRequestContent

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.directory_provisioning.create(
            id="id",
            request=CreateDirectoryProvisioningRequestContent(),
        )
        """
        _response = self._raw_client.create(id, request=request, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete the directory provisioning configuration of a connection.

        Parameters
        ----------
        id : str
            The id of the connection to delete its directory provisioning configuration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.directory_provisioning.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        request: typing.Optional[UpdateDirectoryProvisioningRequestContent] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateDirectoryProvisioningResponseContent:
        """
        Update the directory provisioning configuration of a connection.

        Parameters
        ----------
        id : str
            The id of the connection to create its directory provisioning configuration

        request : typing.Optional[UpdateDirectoryProvisioningRequestContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateDirectoryProvisioningResponseContent
            The connection's directory provisioning configuration was updated. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        from auth0 import Auth0, UpdateDirectoryProvisioningRequestContent

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.directory_provisioning.update(
            id="id",
            request=UpdateDirectoryProvisioningRequestContent(),
        )
        """
        _response = self._raw_client.update(id, request=request, request_options=request_options)
        return _response.data

    def get_default_mapping(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetDirectoryProvisioningDefaultMappingResponseContent:
        """
        Retrieve the directory provisioning default attribute mapping of a connection.

        Parameters
        ----------
        id : str
            The id of the connection to retrieve its directory provisioning configuration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDirectoryProvisioningDefaultMappingResponseContent
            The connection's directory provisioning default mapping. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.directory_provisioning.get_default_mapping(
            id="id",
        )
        """
        _response = self._raw_client.get_default_mapping(id, request_options=request_options)
        return _response.data

    def list_synchronized_groups(
        self,
        id: str,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[SynchronizedGroupPayload, ListSynchronizedGroupsResponseContent]:
        """
        Retrieve the configured synchronized groups for a connection directory provisioning configuration.

        Parameters
        ----------
        id : str
            The id of the connection to list synchronized groups for.

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[SynchronizedGroupPayload, ListSynchronizedGroupsResponseContent]
            The connection's synchronized groups. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.connections.directory_provisioning.list_synchronized_groups(
            id="id",
            from_="from",
            take=1,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list_synchronized_groups(id, from_=from_, take=take, request_options=request_options)

    def set(
        self,
        id: str,
        *,
        groups: typing.Sequence[SynchronizedGroupPayload],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Create or replace the selected groups for a connection directory provisioning configuration.

        Parameters
        ----------
        id : str
            The id of the connection to create or replace synchronized groups for

        groups : typing.Sequence[SynchronizedGroupPayload]
            Array of Google Workspace Directory group objects to synchronize.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0, SynchronizedGroupPayload

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.directory_provisioning.set(
            id="id",
            groups=[
                SynchronizedGroupPayload(
                    id="id",
                )
            ],
        )
        """
        _response = self._raw_client.set(id, groups=groups, request_options=request_options)
        return _response.data

    @property
    def synchronizations(self):
        if self._synchronizations is None:
            from .synchronizations.client import SynchronizationsClient  # noqa: E402

            self._synchronizations = SynchronizationsClient(client_wrapper=self._client_wrapper)
        return self._synchronizations


class AsyncDirectoryProvisioningClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDirectoryProvisioningClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._synchronizations: typing.Optional[AsyncSynchronizationsClient] = None

    @property
    def with_raw_response(self) -> AsyncRawDirectoryProvisioningClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDirectoryProvisioningClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[DirectoryProvisioning, ListDirectoryProvisioningsResponseContent]:
        """
        Retrieve a list of directory provisioning configurations of a tenant.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[DirectoryProvisioning, ListDirectoryProvisioningsResponseContent]
            The tenant's directory provisioning configuration. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.connections.directory_provisioning.list(
                from_="from",
                take=1,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(from_=from_, take=take, request_options=request_options)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetDirectoryProvisioningResponseContent:
        """
        Retrieve the directory provisioning configuration of a connection.

        Parameters
        ----------
        id : str
            The id of the connection to retrieve its directory provisioning configuration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDirectoryProvisioningResponseContent
            The connection's directory provisioning configuration. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.directory_provisioning.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def create(
        self,
        id: str,
        *,
        request: typing.Optional[CreateDirectoryProvisioningRequestContent] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateDirectoryProvisioningResponseContent:
        """
        Create a directory provisioning configuration for a connection.

        Parameters
        ----------
        id : str
            The id of the connection to create its directory provisioning configuration

        request : typing.Optional[CreateDirectoryProvisioningRequestContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateDirectoryProvisioningResponseContent
            The connection's directory provisioning configuration was created. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0, CreateDirectoryProvisioningRequestContent

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.directory_provisioning.create(
                id="id",
                request=CreateDirectoryProvisioningRequestContent(),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(id, request=request, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete the directory provisioning configuration of a connection.

        Parameters
        ----------
        id : str
            The id of the connection to delete its directory provisioning configuration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.directory_provisioning.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        request: typing.Optional[UpdateDirectoryProvisioningRequestContent] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateDirectoryProvisioningResponseContent:
        """
        Update the directory provisioning configuration of a connection.

        Parameters
        ----------
        id : str
            The id of the connection to create its directory provisioning configuration

        request : typing.Optional[UpdateDirectoryProvisioningRequestContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateDirectoryProvisioningResponseContent
            The connection's directory provisioning configuration was updated. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0, UpdateDirectoryProvisioningRequestContent

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.directory_provisioning.update(
                id="id",
                request=UpdateDirectoryProvisioningRequestContent(),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(id, request=request, request_options=request_options)
        return _response.data

    async def get_default_mapping(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetDirectoryProvisioningDefaultMappingResponseContent:
        """
        Retrieve the directory provisioning default attribute mapping of a connection.

        Parameters
        ----------
        id : str
            The id of the connection to retrieve its directory provisioning configuration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDirectoryProvisioningDefaultMappingResponseContent
            The connection's directory provisioning default mapping. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.directory_provisioning.get_default_mapping(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_default_mapping(id, request_options=request_options)
        return _response.data

    async def list_synchronized_groups(
        self,
        id: str,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[SynchronizedGroupPayload, ListSynchronizedGroupsResponseContent]:
        """
        Retrieve the configured synchronized groups for a connection directory provisioning configuration.

        Parameters
        ----------
        id : str
            The id of the connection to list synchronized groups for.

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[SynchronizedGroupPayload, ListSynchronizedGroupsResponseContent]
            The connection's synchronized groups. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.connections.directory_provisioning.list_synchronized_groups(
                id="id",
                from_="from",
                take=1,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list_synchronized_groups(
            id, from_=from_, take=take, request_options=request_options
        )

    async def set(
        self,
        id: str,
        *,
        groups: typing.Sequence[SynchronizedGroupPayload],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Create or replace the selected groups for a connection directory provisioning configuration.

        Parameters
        ----------
        id : str
            The id of the connection to create or replace synchronized groups for

        groups : typing.Sequence[SynchronizedGroupPayload]
            Array of Google Workspace Directory group objects to synchronize.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0, SynchronizedGroupPayload

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.directory_provisioning.set(
                id="id",
                groups=[
                    SynchronizedGroupPayload(
                        id="id",
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.set(id, groups=groups, request_options=request_options)
        return _response.data

    @property
    def synchronizations(self):
        if self._synchronizations is None:
            from .synchronizations.client import AsyncSynchronizationsClient  # noqa: E402

            self._synchronizations = AsyncSynchronizationsClient(client_wrapper=self._client_wrapper)
        return self._synchronizations



# This file was auto-generated by Fern from our API Definition.

from .connection_identity_api_enum_azure_ad import ConnectionIdentityApiEnumAzureAd

ConnectionIdentityApiAzureAd = ConnectionIdentityApiEnumAzureAd
"""
The Azure AD endpoint version for authentication. 'microsoft-identity-platform-v2.0' (recommended, default) supports modern OAuth 2.0 features. 'azure-active-directory-v1.0' is the legacy endpoint with protocol limitations. Selection affects available features.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventOrgConnectionAddedTypeEnum = typing.Union[
    typing.Literal["organization.connection.added"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .event_stream_cloud_event_connection_created_object_3_options_assertion_decryption_settings import (
    EventStreamCloudEventConnectionCreatedObject3OptionsAssertionDecryptionSettings,
)
from .event_stream_cloud_event_connection_created_object_3_options_digest_algorithm_enum import (
    EventStreamCloudEventConnectionCreatedObject3OptionsDigestAlgorithmEnum,
)
from .event_stream_cloud_event_connection_created_object_3_options_idpinitiated import (
    EventStreamCloudEventConnectionCreatedObject3OptionsIdpinitiated,
)
from .event_stream_cloud_event_connection_created_object_3_options_protocol_binding_enum import (
    EventStreamCloudEventConnectionCreatedObject3OptionsProtocolBindingEnum,
)
from .event_stream_cloud_event_connection_created_object_3_options_set_user_root_attributes_enum import (
    EventStreamCloudEventConnectionCreatedObject3OptionsSetUserRootAttributesEnum,
)
from .event_stream_cloud_event_connection_created_object_3_options_signature_algorithm_enum import (
    EventStreamCloudEventConnectionCreatedObject3OptionsSignatureAlgorithmEnum,
)
from .event_stream_cloud_event_connection_created_object_3_options_subject import (
    EventStreamCloudEventConnectionCreatedObject3OptionsSubject,
)
from .event_stream_cloud_event_connection_created_object_3_options_upstream_params import (
    EventStreamCloudEventConnectionCreatedObject3OptionsUpstreamParams,
)


class EventStreamCloudEventConnectionCreatedObject3Options(UniversalBaseModel):
    """
    Options for the 'pingfederate' connection
    """

    assertion_decryption_settings: typing.Optional[
        EventStreamCloudEventConnectionCreatedObject3OptionsAssertionDecryptionSettings
    ] = None
    cert: typing.Optional[str] = pydantic.Field(default=None)
    """
    X.509 signing certificate from the identity provider in .der format. Used to validate signatures in SAML Responses and Assertions. This is an alternative to signingCert and is kept for backward compatibility. Prefer using signingCert instead.
    """

    cert_rollover_notification: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Timestamp of the last certificate expiring soon notification.
    """

    digest_algorithm: typing_extensions.Annotated[
        typing.Optional[EventStreamCloudEventConnectionCreatedObject3OptionsDigestAlgorithmEnum],
        FieldMetadata(alias="digestAlgorithm"),
        pydantic.Field(alias="digestAlgorithm"),
    ] = None
    domain_aliases: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Domain aliases for the connection
    """

    entity_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="entityId"),
        pydantic.Field(
            alias="entityId",
            description="The entity identifier (Issuer) for the SAML Service Provider. When not provided, defaults to 'urn:auth0:{tenant}:{connection}'. This value is included in SAML AuthnRequest messages sent to the identity provider.",
        ),
    ] = None
    """
    The entity identifier (Issuer) for the SAML Service Provider. When not provided, defaults to 'urn:auth0:{tenant}:{connection}'. This value is included in SAML AuthnRequest messages sent to the identity provider.
    """

    expires: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    ISO 8601 formatted datetime indicating when the identity provider's signing certificate expires.
    """

    icon_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL for the connection icon displayed in Auth0 login pages. Accepts HTTPS URLs. Used for visual branding in authentication flows.
    """

    idpinitiated: typing.Optional[EventStreamCloudEventConnectionCreatedObject3OptionsIdpinitiated] = None
    non_persistent_attrs: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    An array of user fields that should not be stored in the Auth0 database (https://auth0.com/docs/security/data-security/denylist)
    """

    protocol_binding: typing_extensions.Annotated[
        typing.Optional[EventStreamCloudEventConnectionCreatedObject3OptionsProtocolBindingEnum],
        FieldMetadata(alias="protocolBinding"),
        pydantic.Field(alias="protocolBinding"),
    ] = None
    set_user_root_attributes: typing.Optional[
        EventStreamCloudEventConnectionCreatedObject3OptionsSetUserRootAttributesEnum
    ] = None
    signature_algorithm: typing_extensions.Annotated[
        typing.Optional[EventStreamCloudEventConnectionCreatedObject3OptionsSignatureAlgorithmEnum],
        FieldMetadata(alias="signatureAlgorithm"),
        pydantic.Field(alias="signatureAlgorithm"),
    ] = None
    sign_in_endpoint: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="signInEndpoint"),
        pydantic.Field(
            alias="signInEndpoint",
            description="Identity provider's SAML SingleSignOnService endpoint URL where Auth0 sends SAML authentication requests. This is the primary login URL for the SAML connection. Required unless using metadataUrl or metadataXml.",
        ),
    ] = None
    """
    Identity provider's SAML SingleSignOnService endpoint URL where Auth0 sends SAML authentication requests. This is the primary login URL for the SAML connection. Required unless using metadataUrl or metadataXml.
    """

    signing_cert: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="signingCert"),
        pydantic.Field(
            alias="signingCert",
            description="Base64-encoded X.509 certificate from the identity provider used to validate signatures in SAML responses and assertions. The certificate is decoded and used for cryptographic signature verification.",
        ),
    ] = None
    """
    Base64-encoded X.509 certificate from the identity provider used to validate signatures in SAML responses and assertions. The certificate is decoded and used for cryptographic signature verification.
    """

    sign_saml_request: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="signSAMLRequest"),
        pydantic.Field(
            alias="signSAMLRequest",
            description="When true, Auth0 signs SAML authentication requests using the connection's signing key. The signature includes the request's digest and is validated by the identity provider. Defaults to false (unsigned requests).",
        ),
    ] = None
    """
    When true, Auth0 signs SAML authentication requests using the connection's signing key. The signature includes the request's digest and is validated by the identity provider. Defaults to false (unsigned requests).
    """

    subject: typing.Optional[EventStreamCloudEventConnectionCreatedObject3OptionsSubject] = None
    tenant_domain: typing.Optional[str] = pydantic.Field(default=None)
    """
    For SAML connections, the tenant domain used to construct the login endpoint URL. Can be a string for single-tenant or an array of strings for multi-tenant validation.
    """

    thumbprints: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    SHA-1 thumbprints (fingerprints) of the identity provider's signing certificates. Automatically computed from signingCert during connection creation. Each thumbprint must be a 40-character hexadecimal string.
    """

    upstream_params: typing.Optional[EventStreamCloudEventConnectionCreatedObject3OptionsUpstreamParams] = None
    ping_federate_base_url: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="pingFederateBaseUrl"),
        pydantic.Field(
            alias="pingFederateBaseUrl",
            description="URL provided by PingFederate which returns information used for creating the connection",
        ),
    ]
    """
    URL provided by PingFederate which returns information used for creating the connection
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_supplementalSignals_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "supplemental_signals.get.0"
    client = get_client(test_id)
    client.supplemental_signals.get()
    verify_request_count(test_id, "GET", "/supplemental-signals", None, 1)


def test_supplementalSignals_patch() -> None:
    """Test patch endpoint with WireMock"""
    test_id = "supplemental_signals.patch.0"
    client = get_client(test_id)
    client.supplemental_signals.patch(
        akamai_enabled=True,
    )
    verify_request_count(test_id, "PATCH", "/supplemental-signals", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .custom_domain_status_filter_enum import CustomDomainStatusFilterEnum
from .custom_domain_type_enum import CustomDomainTypeEnum
from .domain_certificate import DomainCertificate
from .domain_metadata import DomainMetadata
from .domain_verification import DomainVerification


class UpdateDefaultCustomDomainResponseContent(UniversalBaseModel):
    custom_domain_id: str = pydantic.Field()
    """
    ID of the custom domain.
    """

    domain: str = pydantic.Field()
    """
    Domain name.
    """

    primary: bool = pydantic.Field()
    """
    Whether this is a primary domain (true) or not (false).
    """

    is_default: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this is the default custom domain (true) or not (false).
    """

    status: CustomDomainStatusFilterEnum
    type: CustomDomainTypeEnum
    origin_domain_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Intermediate address.
    """

    verification: typing.Optional[DomainVerification] = None
    custom_client_ip_header: typing.Optional[str] = pydantic.Field(default=None)
    """
    The HTTP header to fetch the client's IP address
    """

    tls_policy: typing.Optional[str] = pydantic.Field(default=None)
    """
    The TLS version policy
    """

    domain_metadata: typing.Optional[DomainMetadata] = None
    certificate: typing.Optional[DomainCertificate] = None
    relying_party_identifier: typing.Optional[str] = pydantic.Field(default=None)
    """
    Relying Party ID (rpId) to be used for Passkeys on this custom domain. If not present, the full domain will be used.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.create_self_service_profile_response_content import CreateSelfServiceProfileResponseContent
from ..types.get_self_service_profile_response_content import GetSelfServiceProfileResponseContent
from ..types.list_self_service_profiles_paginated_response_content import (
    ListSelfServiceProfilesPaginatedResponseContent,
)
from ..types.self_service_profile import SelfServiceProfile
from ..types.self_service_profile_allowed_strategy_enum import SelfServiceProfileAllowedStrategyEnum
from ..types.self_service_profile_branding import SelfServiceProfileBranding
from ..types.self_service_profile_branding_properties import SelfServiceProfileBrandingProperties
from ..types.self_service_profile_description import SelfServiceProfileDescription
from ..types.self_service_profile_user_attribute import SelfServiceProfileUserAttribute
from ..types.self_service_profile_user_attributes import SelfServiceProfileUserAttributes
from ..types.update_self_service_profile_response_content import UpdateSelfServiceProfileResponseContent
from .raw_client import AsyncRawSelfServiceProfilesClient, RawSelfServiceProfilesClient

if typing.TYPE_CHECKING:
    from .custom_text.client import AsyncCustomTextClient, CustomTextClient
    from .sso_ticket.client import AsyncSsoTicketClient, SsoTicketClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SelfServiceProfilesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSelfServiceProfilesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._custom_text: typing.Optional[CustomTextClient] = None
        self._sso_ticket: typing.Optional[SsoTicketClient] = None

    @property
    def with_raw_response(self) -> RawSelfServiceProfilesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSelfServiceProfilesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[SelfServiceProfile, ListSelfServiceProfilesPaginatedResponseContent]:
        """
        Retrieves self-service profiles.

        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[SelfServiceProfile, ListSelfServiceProfilesPaginatedResponseContent]
            List of existing profiles.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.self_service_profiles.list(
            page=1,
            per_page=1,
            include_totals=True,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            page=page, per_page=per_page, include_totals=include_totals, request_options=request_options
        )

    def create(
        self,
        *,
        name: str,
        description: typing.Optional[str] = OMIT,
        branding: typing.Optional[SelfServiceProfileBrandingProperties] = OMIT,
        allowed_strategies: typing.Optional[typing.Sequence[SelfServiceProfileAllowedStrategyEnum]] = OMIT,
        user_attributes: typing.Optional[typing.Sequence[SelfServiceProfileUserAttribute]] = OMIT,
        user_attribute_profile_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateSelfServiceProfileResponseContent:
        """
        Creates a self-service profile.

        Parameters
        ----------
        name : str
            The name of the self-service Profile.

        description : typing.Optional[str]
            The description of the self-service Profile.

        branding : typing.Optional[SelfServiceProfileBrandingProperties]

        allowed_strategies : typing.Optional[typing.Sequence[SelfServiceProfileAllowedStrategyEnum]]
            List of IdP strategies that will be shown to users during the Self-Service Enterprise Configuration flow. Possible values: [`oidc`, `samlp`, `waad`, `google-apps`, `adfs`, `okta`, `auth0-samlp`, `okta-samlp`, `keycloak-samlp`, `pingfederate`]

        user_attributes : typing.Optional[typing.Sequence[SelfServiceProfileUserAttribute]]
            List of attributes to be mapped that will be shown to the user during the Self-Service Enterprise Configuration flow.

        user_attribute_profile_id : typing.Optional[str]
            ID of the user-attribute-profile to associate with this self-service profile.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateSelfServiceProfileResponseContent
            Self-service profile successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.self_service_profiles.create(
            name="name",
        )
        """
        _response = self._raw_client.create(
            name=name,
            description=description,
            branding=branding,
            allowed_strategies=allowed_strategies,
            user_attributes=user_attributes,
            user_attribute_profile_id=user_attribute_profile_id,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetSelfServiceProfileResponseContent:
        """
        Retrieves a self-service profile by Id.

        Parameters
        ----------
        id : str
            The id of the self-service profile to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSelfServiceProfileResponseContent
            Record for existing self-service profile.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.self_service_profiles.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Deletes a self-service profile by Id.

        Parameters
        ----------
        id : str
            The id of the self-service profile to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.self_service_profiles.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[SelfServiceProfileDescription] = OMIT,
        branding: typing.Optional[SelfServiceProfileBranding] = OMIT,
        allowed_strategies: typing.Optional[typing.Sequence[SelfServiceProfileAllowedStrategyEnum]] = OMIT,
        user_attributes: typing.Optional[SelfServiceProfileUserAttributes] = OMIT,
        user_attribute_profile_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateSelfServiceProfileResponseContent:
        """
        Updates a self-service profile.

        Parameters
        ----------
        id : str
            The id of the self-service profile to update

        name : typing.Optional[str]
            The name of the self-service Profile.

        description : typing.Optional[SelfServiceProfileDescription]

        branding : typing.Optional[SelfServiceProfileBranding]

        allowed_strategies : typing.Optional[typing.Sequence[SelfServiceProfileAllowedStrategyEnum]]
            List of IdP strategies that will be shown to users during the Self-Service Enterprise Configuration flow. Possible values: [`oidc`, `samlp`, `waad`, `google-apps`, `adfs`, `okta`, `auth0-samlp`, `okta-samlp`, `keycloak-samlp`, `pingfederate`]

        user_attributes : typing.Optional[SelfServiceProfileUserAttributes]

        user_attribute_profile_id : typing.Optional[str]
            ID of the user-attribute-profile to associate with this self-service profile.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateSelfServiceProfileResponseContent
            Self-service profile successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.self_service_profiles.update(
            id="id",
        )
        """
        _response = self._raw_client.update(
            id,
            name=name,
            description=description,
            branding=branding,
            allowed_strategies=allowed_strategies,
            user_attributes=user_attributes,
            user_attribute_profile_id=user_attribute_profile_id,
            request_options=request_options,
        )
        return _response.data

    @property
    def custom_text(self):
        if self._custom_text is None:
            from .custom_text.client import CustomTextClient  # noqa: E402

            self._custom_text = CustomTextClient(client_wrapper=self._client_wrapper)
        return self._custom_text

    @property
    def sso_ticket(self):
        if self._sso_ticket is None:
            from .sso_ticket.client import SsoTicketClient  # noqa: E402

            self._sso_ticket = SsoTicketClient(client_wrapper=self._client_wrapper)
        return self._sso_ticket


class AsyncSelfServiceProfilesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSelfServiceProfilesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._custom_text: typing.Optional[AsyncCustomTextClient] = None
        self._sso_ticket: typing.Optional[AsyncSsoTicketClient] = None

    @property
    def with_raw_response(self) -> AsyncRawSelfServiceProfilesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSelfServiceProfilesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[SelfServiceProfile, ListSelfServiceProfilesPaginatedResponseContent]:
        """
        Retrieves self-service profiles.

        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[SelfServiceProfile, ListSelfServiceProfilesPaginatedResponseContent]
            List of existing profiles.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.self_service_profiles.list(
                page=1,
                per_page=1,
                include_totals=True,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            page=page, per_page=per_page, include_totals=include_totals, request_options=request_options
        )

    async def create(
        self,
        *,
        name: str,
        description: typing.Optional[str] = OMIT,
        branding: typing.Optional[SelfServiceProfileBrandingProperties] = OMIT,
        allowed_strategies: typing.Optional[typing.Sequence[SelfServiceProfileAllowedStrategyEnum]] = OMIT,
        user_attributes: typing.Optional[typing.Sequence[SelfServiceProfileUserAttribute]] = OMIT,
        user_attribute_profile_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateSelfServiceProfileResponseContent:
        """
        Creates a self-service profile.

        Parameters
        ----------
        name : str
            The name of the self-service Profile.

        description : typing.Optional[str]
            The description of the self-service Profile.

        branding : typing.Optional[SelfServiceProfileBrandingProperties]

        allowed_strategies : typing.Optional[typing.Sequence[SelfServiceProfileAllowedStrategyEnum]]
            List of IdP strategies that will be shown to users during the Self-Service Enterprise Configuration flow. Possible values: [`oidc`, `samlp`, `waad`, `google-apps`, `adfs`, `okta`, `auth0-samlp`, `okta-samlp`, `keycloak-samlp`, `pingfederate`]

        user_attributes : typing.Optional[typing.Sequence[SelfServiceProfileUserAttribute]]
            List of attributes to be mapped that will be shown to the user during the Self-Service Enterprise Configuration flow.

        user_attribute_profile_id : typing.Optional[str]
            ID of the user-attribute-profile to associate with this self-service profile.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateSelfServiceProfileResponseContent
            Self-service profile successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.self_service_profiles.create(
                name="name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name,
            description=description,
            branding=branding,
            allowed_strategies=allowed_strategies,
            user_attributes=user_attributes,
            user_attribute_profile_id=user_attribute_profile_id,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetSelfServiceProfileResponseContent:
        """
        Retrieves a self-service profile by Id.

        Parameters
        ----------
        id : str
            The id of the self-service profile to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSelfServiceProfileResponseContent
            Record for existing self-service profile.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.self_service_profiles.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Deletes a self-service profile by Id.

        Parameters
        ----------
        id : str
            The id of the self-service profile to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.self_service_profiles.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[SelfServiceProfileDescription] = OMIT,
        branding: typing.Optional[SelfServiceProfileBranding] = OMIT,
        allowed_strategies: typing.Optional[typing.Sequence[SelfServiceProfileAllowedStrategyEnum]] = OMIT,
        user_attributes: typing.Optional[SelfServiceProfileUserAttributes] = OMIT,
        user_attribute_profile_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateSelfServiceProfileResponseContent:
        """
        Updates a self-service profile.

        Parameters
        ----------
        id : str
            The id of the self-service profile to update

        name : typing.Optional[str]
            The name of the self-service Profile.

        description : typing.Optional[SelfServiceProfileDescription]

        branding : typing.Optional[SelfServiceProfileBranding]

        allowed_strategies : typing.Optional[typing.Sequence[SelfServiceProfileAllowedStrategyEnum]]
            List of IdP strategies that will be shown to users during the Self-Service Enterprise Configuration flow. Possible values: [`oidc`, `samlp`, `waad`, `google-apps`, `adfs`, `okta`, `auth0-samlp`, `okta-samlp`, `keycloak-samlp`, `pingfederate`]

        user_attributes : typing.Optional[SelfServiceProfileUserAttributes]

        user_attribute_profile_id : typing.Optional[str]
            ID of the user-attribute-profile to associate with this self-service profile.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateSelfServiceProfileResponseContent
            Self-service profile successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.self_service_profiles.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id,
            name=name,
            description=description,
            branding=branding,
            allowed_strategies=allowed_strategies,
            user_attributes=user_attributes,
            user_attribute_profile_id=user_attribute_profile_id,
            request_options=request_options,
        )
        return _response.data

    @property
    def custom_text(self):
        if self._custom_text is None:
            from .custom_text.client import AsyncCustomTextClient  # noqa: E402

            self._custom_text = AsyncCustomTextClient(client_wrapper=self._client_wrapper)
        return self._custom_text

    @property
    def sso_ticket(self):
        if self._sso_ticket is None:
            from .sso_ticket.client import AsyncSsoTicketClient  # noqa: E402

            self._sso_ticket = AsyncSsoTicketClient(client_wrapper=self._client_wrapper)
        return self._sso_ticket



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from ....types.create_directory_synchronization_response_content import CreateDirectorySynchronizationResponseContent
from .raw_client import AsyncRawSynchronizationsClient, RawSynchronizationsClient


class SynchronizationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSynchronizationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSynchronizationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSynchronizationsClient
        """
        return self._raw_client

    def create(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateDirectorySynchronizationResponseContent:
        """
        Request an on-demand synchronization of the directory.

        Parameters
        ----------
        id : str
            The id of the connection to trigger synchronization for

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateDirectorySynchronizationResponseContent
            The directory synchronization was triggered. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.directory_provisioning.synchronizations.create(
            id="id",
        )
        """
        _response = self._raw_client.create(id, request_options=request_options)
        return _response.data


class AsyncSynchronizationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSynchronizationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSynchronizationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSynchronizationsClient
        """
        return self._raw_client

    async def create(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateDirectorySynchronizationResponseContent:
        """
        Request an on-demand synchronization of the directory.

        Parameters
        ----------
        id : str
            The id of the connection to trigger synchronization for

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateDirectorySynchronizationResponseContent
            The directory synchronization was triggered. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.directory_provisioning.synchronizations.create(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.bot_detection_allowlist import BotDetectionAllowlist
from ...types.bot_detection_challenge_policy_password_flow_enum import BotDetectionChallengePolicyPasswordFlowEnum
from ...types.bot_detection_challenge_policy_password_reset_flow_enum import (
    BotDetectionChallengePolicyPasswordResetFlowEnum,
)
from ...types.bot_detection_challenge_policy_passwordless_flow_enum import (
    BotDetectionChallengePolicyPasswordlessFlowEnum,
)
from ...types.bot_detection_level_enum import BotDetectionLevelEnum
from ...types.bot_detection_monitoring_mode_enabled import BotDetectionMonitoringModeEnabled
from ...types.get_bot_detection_settings_response_content import GetBotDetectionSettingsResponseContent
from ...types.update_bot_detection_settings_response_content import UpdateBotDetectionSettingsResponseContent
from .raw_client import AsyncRawBotDetectionClient, RawBotDetectionClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class BotDetectionClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawBotDetectionClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawBotDetectionClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawBotDetectionClient
        """
        return self._raw_client

    def get(self, *, request_options: typing.Optional[RequestOptions] = None) -> GetBotDetectionSettingsResponseContent:
        """
        Get the Bot Detection configuration of your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBotDetectionSettingsResponseContent
            Bot detection configuration retrieved successfully.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.attack_protection.bot_detection.get()
        """
        _response = self._raw_client.get(request_options=request_options)
        return _response.data

    def update(
        self,
        *,
        bot_detection_level: typing.Optional[BotDetectionLevelEnum] = OMIT,
        challenge_password_policy: typing.Optional[BotDetectionChallengePolicyPasswordFlowEnum] = OMIT,
        challenge_passwordless_policy: typing.Optional[BotDetectionChallengePolicyPasswordlessFlowEnum] = OMIT,
        challenge_password_reset_policy: typing.Optional[BotDetectionChallengePolicyPasswordResetFlowEnum] = OMIT,
        allowlist: typing.Optional[BotDetectionAllowlist] = OMIT,
        monitoring_mode_enabled: typing.Optional[BotDetectionMonitoringModeEnabled] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateBotDetectionSettingsResponseContent:
        """
        Update the Bot Detection configuration of your tenant.

        Parameters
        ----------
        bot_detection_level : typing.Optional[BotDetectionLevelEnum]

        challenge_password_policy : typing.Optional[BotDetectionChallengePolicyPasswordFlowEnum]

        challenge_passwordless_policy : typing.Optional[BotDetectionChallengePolicyPasswordlessFlowEnum]

        challenge_password_reset_policy : typing.Optional[BotDetectionChallengePolicyPasswordResetFlowEnum]

        allowlist : typing.Optional[BotDetectionAllowlist]

        monitoring_mode_enabled : typing.Optional[BotDetectionMonitoringModeEnabled]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateBotDetectionSettingsResponseContent
            Bot detection configuration successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.attack_protection.bot_detection.update()
        """
        _response = self._raw_client.update(
            bot_detection_level=bot_detection_level,
            challenge_password_policy=challenge_password_policy,
            challenge_passwordless_policy=challenge_passwordless_policy,
            challenge_password_reset_policy=challenge_password_reset_policy,
            allowlist=allowlist,
            monitoring_mode_enabled=monitoring_mode_enabled,
            request_options=request_options,
        )
        return _response.data


class AsyncBotDetectionClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawBotDetectionClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawBotDetectionClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawBotDetectionClient
        """
        return self._raw_client

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBotDetectionSettingsResponseContent:
        """
        Get the Bot Detection configuration of your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBotDetectionSettingsResponseContent
            Bot detection configuration retrieved successfully.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.attack_protection.bot_detection.get()


        asyncio.run(main())
        """
        _response = await self._raw_client.get(request_options=request_options)
        return _response.data

    async def update(
        self,
        *,
        bot_detection_level: typing.Optional[BotDetectionLevelEnum] = OMIT,
        challenge_password_policy: typing.Optional[BotDetectionChallengePolicyPasswordFlowEnum] = OMIT,
        challenge_passwordless_policy: typing.Optional[BotDetectionChallengePolicyPasswordlessFlowEnum] = OMIT,
        challenge_password_reset_policy: typing.Optional[BotDetectionChallengePolicyPasswordResetFlowEnum] = OMIT,
        allowlist: typing.Optional[BotDetectionAllowlist] = OMIT,
        monitoring_mode_enabled: typing.Optional[BotDetectionMonitoringModeEnabled] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateBotDetectionSettingsResponseContent:
        """
        Update the Bot Detection configuration of your tenant.

        Parameters
        ----------
        bot_detection_level : typing.Optional[BotDetectionLevelEnum]

        challenge_password_policy : typing.Optional[BotDetectionChallengePolicyPasswordFlowEnum]

        challenge_passwordless_policy : typing.Optional[BotDetectionChallengePolicyPasswordlessFlowEnum]

        challenge_password_reset_policy : typing.Optional[BotDetectionChallengePolicyPasswordResetFlowEnum]

        allowlist : typing.Optional[BotDetectionAllowlist]

        monitoring_mode_enabled : typing.Optional[BotDetectionMonitoringModeEnabled]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateBotDetectionSettingsResponseContent
            Bot detection configuration successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.attack_protection.bot_detection.update()


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            bot_detection_level=bot_detection_level,
            challenge_password_policy=challenge_password_policy,
            challenge_passwordless_policy=challenge_passwordless_policy,
            challenge_password_reset_policy=challenge_password_reset_policy,
            allowlist=allowlist,
            monitoring_mode_enabled=monitoring_mode_enabled,
            request_options=request_options,
        )
        return _response.data



from .conftest import get_client, verify_request_count


def test_prompts_get_settings() -> None:
    """Test getSettings endpoint with WireMock"""
    test_id = "prompts.get_settings.0"
    client = get_client(test_id)
    client.prompts.get_settings()
    verify_request_count(test_id, "GET", "/prompts", None, 1)


def test_prompts_update_settings() -> None:
    """Test updateSettings endpoint with WireMock"""
    test_id = "prompts.update_settings.0"
    client = get_client(test_id)
    client.prompts.update_settings()
    verify_request_count(test_id, "PATCH", "/prompts", None, 1)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .integration_feature_type_enum import IntegrationFeatureTypeEnum
from .integration_release import IntegrationRelease


class Integration(UniversalBaseModel):
    """
    Integration defines a self contained functioning unit which partners
    publish. A partner may create one or many of these integrations.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    id is a system generated GUID. This same ID is designed to be federated in
    all the applicable localities.
    """

    catalog_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    catalog_id refers to the ID in the marketplace catalog
    """

    url_slug: typing.Optional[str] = pydantic.Field(default=None)
    """
    url_slug refers to the url_slug in the marketplace catalog
    """

    partner_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    partner_id is the foreign key reference to the partner account this
    integration belongs to.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    name is the integration name, which will be used for display purposes in
    the marketplace.
    
    To start we're going to make sure the display name is at least 3
    characters. Can adjust this easily later.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    description adds more text for the integration name -- also relevant for
    the marketplace listing.
    """

    short_description: typing.Optional[str] = pydantic.Field(default=None)
    """
    short_description is the brief description of the integration, which is used for display purposes in cards
    """

    logo: typing.Optional[str] = None
    feature_type: typing.Optional[IntegrationFeatureTypeEnum] = None
    terms_of_use_url: typing.Optional[str] = None
    privacy_policy_url: typing.Optional[str] = None
    public_support_link: typing.Optional[str] = None
    current_release: typing.Optional[IntegrationRelease] = None
    created_at: typing.Optional[dt.datetime] = None
    updated_at: typing.Optional[dt.datetime] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_users_identities_link() -> None:
    """Test link endpoint with WireMock"""
    test_id = "users.identities.link.0"
    client = get_client(test_id)
    client.users.identities.link(
        id="id",
    )
    verify_request_count(test_id, "POST", "/users/id/identities", None, 1)


def test_users_identities_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "users.identities.delete.0"
    client = get_client(test_id)
    client.users.identities.delete(
        id="id",
        provider="ad",
        user_id="user_id",
    )
    verify_request_count(test_id, "DELETE", "/users/id/identities/ad/user_id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.http_response import AsyncHttpResponse, HttpResponse
from .....core.parse_error import ParsingError
from .....core.pydantic_utilities import parse_obj_as
from .....core.request_options import RequestOptions
from .....errors.bad_request_error import BadRequestError
from .....errors.forbidden_error import ForbiddenError
from .....errors.unauthorized_error import UnauthorizedError
from .....types.get_guardian_factor_duo_settings_response_content import GetGuardianFactorDuoSettingsResponseContent
from .....types.set_guardian_factor_duo_settings_response_content import SetGuardianFactorDuoSettingsResponseContent
from .....types.update_guardian_factor_duo_settings_response_content import (
    UpdateGuardianFactorDuoSettingsResponseContent,
)
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSettingsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetGuardianFactorDuoSettingsResponseContent]:
        """
        Retrieves the DUO account and factor configuration.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetGuardianFactorDuoSettingsResponseContent]
            DUO settings successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "guardian/factors/duo/settings",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetGuardianFactorDuoSettingsResponseContent,
                    parse_obj_as(
                        type_=GetGuardianFactorDuoSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def set(
        self,
        *,
        ikey: typing.Optional[str] = OMIT,
        skey: typing.Optional[str] = OMIT,
        host: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SetGuardianFactorDuoSettingsResponseContent]:
        """
        Set the DUO account configuration and other properties specific to this factor.

        Parameters
        ----------
        ikey : typing.Optional[str]

        skey : typing.Optional[str]

        host : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SetGuardianFactorDuoSettingsResponseContent]
            DUO settings successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            "guardian/factors/duo/settings",
            method="PUT",
            json={
                "ikey": ikey,
                "skey": skey,
                "host": host,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SetGuardianFactorDuoSettingsResponseContent,
                    parse_obj_as(
                        type_=SetGuardianFactorDuoSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        *,
        ikey: typing.Optional[str] = OMIT,
        skey: typing.Optional[str] = OMIT,
        host: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateGuardianFactorDuoSettingsResponseContent]:
        """
        Parameters
        ----------
        ikey : typing.Optional[str]

        skey : typing.Optional[str]

        host : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateGuardianFactorDuoSettingsResponseContent]
            DUO settings successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            "guardian/factors/duo/settings",
            method="PATCH",
            json={
                "ikey": ikey,
                "skey": skey,
                "host": host,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateGuardianFactorDuoSettingsResponseContent,
                    parse_obj_as(
                        type_=UpdateGuardianFactorDuoSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSettingsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetGuardianFactorDuoSettingsResponseContent]:
        """
        Retrieves the DUO account and factor configuration.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetGuardianFactorDuoSettingsResponseContent]
            DUO settings successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guardian/factors/duo/settings",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetGuardianFactorDuoSettingsResponseContent,
                    parse_obj_as(
                        type_=GetGuardianFactorDuoSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def set(
        self,
        *,
        ikey: typing.Optional[str] = OMIT,
        skey: typing.Optional[str] = OMIT,
        host: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SetGuardianFactorDuoSettingsResponseContent]:
        """
        Set the DUO account configuration and other properties specific to this factor.

        Parameters
        ----------
        ikey : typing.Optional[str]

        skey : typing.Optional[str]

        host : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SetGuardianFactorDuoSettingsResponseContent]
            DUO settings successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guardian/factors/duo/settings",
            method="PUT",
            json={
                "ikey": ikey,
                "skey": skey,
                "host": host,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SetGuardianFactorDuoSettingsResponseContent,
                    parse_obj_as(
                        type_=SetGuardianFactorDuoSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        *,
        ikey: typing.Optional[str] = OMIT,
        skey: typing.Optional[str] = OMIT,
        host: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateGuardianFactorDuoSettingsResponseContent]:
        """
        Parameters
        ----------
        ikey : typing.Optional[str]

        skey : typing.Optional[str]

        host : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateGuardianFactorDuoSettingsResponseContent]
            DUO settings successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guardian/factors/duo/settings",
            method="PATCH",
            json={
                "ikey": ikey,
                "skey": skey,
                "host": host,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateGuardianFactorDuoSettingsResponseContent,
                    parse_obj_as(
                        type_=UpdateGuardianFactorDuoSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .connection_name_prefix_template import ConnectionNamePrefixTemplate
from .connection_profile_config import ConnectionProfileConfig
from .connection_profile_enabled_features import ConnectionProfileEnabledFeatures
from .connection_profile_id import ConnectionProfileId
from .connection_profile_name import ConnectionProfileName
from .connection_profile_organization import ConnectionProfileOrganization
from .connection_profile_strategy_overrides import ConnectionProfileStrategyOverrides


class UpdateConnectionProfileResponseContent(UniversalBaseModel):
    id: typing.Optional[ConnectionProfileId] = None
    name: typing.Optional[ConnectionProfileName] = None
    organization: typing.Optional[ConnectionProfileOrganization] = None
    connection_name_prefix_template: typing.Optional[ConnectionNamePrefixTemplate] = None
    enabled_features: typing.Optional[ConnectionProfileEnabledFeatures] = None
    connection_config: typing.Optional[ConnectionProfileConfig] = None
    strategy_overrides: typing.Optional[ConnectionProfileStrategyOverrides] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ResourceServerTokenEncryptionAlgorithmEnum = typing.Union[
    typing.Literal["RSA-OAEP-256", "RSA-OAEP-384", "RSA-OAEP-512"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .acul_organization_filter_by_id import AculOrganizationFilterById
from .acul_organization_filter_by_metadata import AculOrganizationFilterByMetadata

AculOrganizationFilter = typing.Union[AculOrganizationFilterById, AculOrganizationFilterByMetadata]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .email_attribute import EmailAttribute
from .phone_attribute import PhoneAttribute
from .username_attribute import UsernameAttribute


class ConnectionAttributes(UniversalBaseModel):
    """
    Attribute configuration
    """

    email: typing.Optional[EmailAttribute] = None
    phone_number: typing.Optional[PhoneAttribute] = None
    username: typing.Optional[UsernameAttribute] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_stream_cloud_event_group_role_assigned_object_group import EventStreamCloudEventGroupRoleAssignedObjectGroup
from .event_stream_cloud_event_group_role_assigned_object_role import EventStreamCloudEventGroupRoleAssignedObjectRole


class EventStreamCloudEventGroupRoleAssignedObject(UniversalBaseModel):
    """
    The event content.
    """

    group: EventStreamCloudEventGroupRoleAssignedObjectGroup
    role: EventStreamCloudEventGroupRoleAssignedObjectRole
    created_at: dt.datetime = pydantic.Field()
    """
    The time at which the role was assigned to the group.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count

from auth0.management import EmailProviderCredentialsSchemaZero


def test_emails_provider_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "emails.provider.get.0"
    client = get_client(test_id)
    client.emails.provider.get(
        fields="fields",
        include_fields=True,
    )
    verify_request_count(test_id, "GET", "/emails/provider", {"fields": "fields", "include_fields": "true"}, 1)


def test_emails_provider_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "emails.provider.create.0"
    client = get_client(test_id)
    client.emails.provider.create(
        name="mailgun",
        credentials=EmailProviderCredentialsSchemaZero(
            api_key="api_key",
        ),
    )
    verify_request_count(test_id, "POST", "/emails/provider", None, 1)


def test_emails_provider_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "emails.provider.delete.0"
    client = get_client(test_id)
    client.emails.provider.delete()
    verify_request_count(test_id, "DELETE", "/emails/provider", None, 1)


def test_emails_provider_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "emails.provider.update.0"
    client = get_client(test_id)
    client.emails.provider.update()
    verify_request_count(test_id, "PATCH", "/emails/provider", None, 1)
